
/*
 * Copyright (c) 2004, INRIA
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 
 * 	- Redistributions of source code must retain the above copyright
 * 	notice, this list of conditions and the following disclaimer.  
 * 	- Redistributions in binary form must reproduce the above copyright
 * 	notice, this list of conditions and the following disclaimer in the
 * 	documentation and/or other materials provided with the distribution.
 * 	- Neither the name of the INRIA nor the names of its
 * 	contributors may be used to endorse or promote products derived from
 * 	this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


import tom.library.xml.*;
import tom.library.adt.tnode.*;
import tom.library.adt.tnode.types.*;
import java.io.*;
import java.util.Hashtable;
import java.util.LinkedList;
import aterm.*;
import tom.library.traversal.*;

public class MHTMLc {

	%include{ adt/TNode.tom }

	private XmlTools xtools, xtools2;
	private GenericTraversal traversal = new GenericTraversal();
	private FileWriter out;
	//private final boolean rootModuleFound = false;
	private static Hashtable table = new Hashtable();
	private static LinkedList fpList = new LinkedList();
	private final String initial = "InitialEntry";
	private TNodeFactory getTNodeFactory() {
		return xtools.getTNodeFactory();
	}

	public static void main(String[] args) {
		MHTMLc mhtml = new MHTMLc();
		if (args.length == 2) {
			mhtml.run(args[0],args[1]);
		} else {
			System.out.println("Usage : java MHTML file.mhtml output.html\n");
		}
	}
	private void write(String s) {
		try {
			out.write(s);
		} catch (IOException e) {
			System.err.println("Error in write");
		}
	}

	private void run(String filename, String output){
		try {
			out = new FileWriter(output);
			xtools = new XmlTools();
			xtools2 = new XmlTools();
			
			char doublequote = '"';
			write("<!DOCTYPE html   PUBLIC " + doublequote + "-//W3C//DTD XHTML 1.0 Strict//EN" + doublequote +
				"\n" + doublequote +"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"+ doublequote + ">\n");

			//testing if the document is fine
		
			try
			{	//an exception should be thrown here... but it is not!
				TNode test =  (TNode) xtools.convertXMLToATerm(filename);
				test.getDocElem();

			}catch(Exception e){
				System.err.println("ERROR: The document presents a markup problem, exiting.");
				System.exit(1);
			}
			ATerm term  =  xtools.convertXMLToATerm(filename);
			ATerm term1;// =  xtools.convertXMLToATerm(output);


			term1 = translate(term);
			xtools2.writeXMLFileFromATerm(out,term1);			
			out.flush();
			out.close();
		} catch (IOException e) {
			System.err.println("Cannot open file "+output);
		}
	}


	private ATerm translate(ATerm subject) {

		Replace1 replace = new Replace1 () {
			public ATerm apply(ATerm t) {
				if (t instanceof TNode) {
					%match(TNode t) {
						
							           
							<module>body@<html></html></module> 	|
							<module>body@<head></head> </module>	|
							<module>body@<title></title></module>	|
							<module>body@<base></base></module>	|
							<module>body@<meta></meta></module>	|
							<module>body@<link></link></module>	|
							<module>body@<style></style></module>	|
							<module>body@<script></script></module>	|
							<module>body@<noscript></noscript></module>	|
							<module>body@<body></body></module>	|
							<module>body@<div></div></module> 	|
							<module>body@<p></p></module>	|
							<module>body@<h1></h1></module>		|
							<module>body@<h2></h2></module>		|
							<module>body@<h3></h3></module>		|
							<module>body@<h4></h4></module>		|
							<module>body@<h5></h5></module>		|
							<module>body@<h6></h6></module>		|
							<module>body@<ul></ul></module>		|
							<module>body@<ol></ol></module>		|
							<module>body@<li></li> </module>	|	
							<module>body@<dl></dl></module>		|
							<module>body@<dt></dt></module>		|
							<module>body@<dd></dd></module>		|
							<module>body@<address></address></module>	|
							<module>body@<hr></hr></module>		|
							<module>body@<pre></pre></module>	|
							<module>body@<blockquote></blockquote></module>	|
							<module>body@<ins></ins></module>	|
							<module>body@<del></del></module>	|
							<module>body@<a></a></module>		|
							<module>body@<span></span> </module>	|
							<module>body@<bdo></bdo></module> 	|
							<module>body@<br></br></module>   	|
							<module>body@<em></em></module> 	|
							<module>body@<strong></strong></module> | 
							<module>body@<dfn></dfn></module>   	|
							<module>body@<code></code></module>  	|
							<module>body@<samp></samp></module> 	|
							<module>body@<kbd></kbd></module>  	|
							<module>body@<var></var></module>  	|
							<module>body@<cite></cite></module> 	|
							<module>body@<abbr></abbr></module> 	|
							<module>body@<acronym></acronym></module>	| 
							<module>body@<q></q></module> 		|
							<module>body@<sub></sub></module> 	|
							<module>body@<sup></sup></module> 	|
							<module>body@<tt></tt></module>  	|
							<module>body@<i></i></module>   	|
							<module>body@<b></b></module>   	|
							<module>body@<big></big></module> 	|
							<module>body@<small></small></module> 	|
							<module>body@<object></object></module>	|
							<module>body@<param></param></module>	|
							<module>body@<img></img></module>	|
							<module>body@<map></map></module>	|
							<module>body@<area></area></module>	|
							<module>body@<form></form></module>  	|
							<module>body@<label></label></module>	|
							<module>body@<input></input></module>   |
							<module>body@<select></select></module> |
							<module>body@<optgroup></optgroup></module>  |
							<module>body@<option></option></module> |
							<module>body@<textarea></textarea></module>  |
							<module>body@<fieldset></fieldset></module>  |
							<module>body@<legend></legend></module>	|
							<module>body@<button></button></module> |
							<module>body@<table></table></module>	|
							<module>body@<caption></caption></module>    |
							<module>body@<thead></thead></module>	|
							<module>body@<tfoot></tfoot></module>	|
							<module>body@<tbody></tbody></module>	|
							<module>body@<colgroup></colgroup></module>  |
							<module>body@<col></col></module>	|
							<module>body@<tr></tr></module>		|
							<module>body@<th></th></module>		|
							<module>body@<td></td></module>		|
							
							<module>body@<instantiate></instantiate></module> |  
							<module>body@<import></import></module>	|	
							<module>body@<params></params></module>		
							->  {
								return translate(`body); 	
							}
						imp@<import>#TEXT(impName)</import>   -> {

							XmlTools localXtool = new XmlTools();
							
							ATerm node = localXtool.convertXMLToATerm(`impName.trim()+".mhtml");
							node = translate(node);
							return node; 
						}

						/* I decided to put this markup to differ from regular importations */

						imp@<instantiate>#TEXT(impName)</instantiate>   -> {
							/* Here I have to read the param bindings to build the
								 symbol table, and after read the  template   */
							/* Here I have to visit the children */
							%match(TNode t){
								<instantiate>acParam@<actualparam fp=fpName>#TEXT(apName)</actualparam></instantiate>   -> {
								
									XmlTools localXtool = new XmlTools();
									ATerm node = localXtool.convertXMLToATerm(`apName.trim()+".mhtml");
									node = translate(node);
									/* Here I need to record really the string in the table...  */
									table.put(`fpName.toString().trim(),node);
									/* return traversal.genericTraversal(t,this);*/
								}
								<instantiate>variable@<importName>#TEXT(templ)</importName></instantiate>   -> {
									XmlTools localXtool = new XmlTools();
									ATerm node = localXtool.convertXMLToATerm(`templ.trim()+".mhtml");
									return translate(node);

								}

								_ -> {
									/* This is possibly equivalent to the second rule */
									return traversal.genericTraversal(t,this);
								}
							}
						}

						/* This matches formal parameter declarations   */
						<params>par@<param>#TEXT(fpName)</param></params> -> {
							fpList.add(`fpName.trim());
							if (!table.containsKey(`fpName.trim())) {
								
								System.err.println("ERROR: All parameters must be instantiated.");
								System.exit(1);
							} else {
								System.out.println("The param "+ `fpName + " has been instantiated");
								return  traversal.genericTraversal(t,this);
							}
						}		

						use@<use>#TEXT(paramName)</use>   -> {
							/* Here I should retrieve the tree in the symbol table */
							/* using the paramName as a key */
							/* Here I should return the value from the table to be replaced in the tree */
							ATerm myNode;
							try{ 
								myNode = (ATerm) table.get(`paramName.toString().trim());
								
								if (myNode != null){
									return myNode; 
								}
							}
							catch(NullPointerException e){
								System.err.println("ERROR: entry does not exist in the table: " +`paramName);
								System.exit(1);
							}

						}


						_ -> {
							/* This is possibly equivalent to the second rule */
							return traversal.genericTraversal(t,this);
						}
					} // match 
					} else {
						return traversal.genericTraversal(t, this);
					}
				} //apply
			};

			return replace.apply(subject);
		} //translate
	}
